@@ -53,7 +53,8 @@ static int do_recv(int sock, char *buffer, int size) {
 		return -1;
 	while (len > 0 && (buffer[len - 1] == '\r' || buffer[len - 1] == '\n'))
 		len--;
-	buffer[len] = '\0';
+	if (len == size)
+		buffer[len - 1] = '\0';
 
 	if (v_opt)
 		log("<-- [%s]", buffer);
@@ -99,7 +100,6 @@ void clean_pathname(char *pathname)
 				} else
 					break;
 			}
-		c = '/';
 		}
 		*cleanpath++ = c;
 	}
@@ -220,7 +220,7 @@ static void handle_size(int sock, char *request)
       		return;
 	}
 
- 	snprintf(response, sizeof(response), "%015ld", st.st_size);
+ 	snprintf(response, sizeof(response), "%015lld", st.st_size);
   	do_send(sock, response, 15);
 }
 
@@ -232,6 +232,7 @@ static void handle_get(int sock, char *request, int fd)
 	size_t chunk;
 	int lfd = -1;
 	long o, c;
+	size_t sent;
 
 	p1 = index(request, '/');
 	p2 = index(request, '|');
@@ -280,9 +281,9 @@ static void handle_get(int sock, char *request, int fd)
 	}
 
 #ifdef Linux
-	if (sendfile(sock, fd, &offset, chunk) < 0) {
+	if ((sent = sendfile(sock, fd, &offset, chunk)) < 0) {
 #else /* FreeBSD */
-	if (sendfile(fd, sock, offset, chunk, NULL, NULL, 0) < 0) {
+	if ((sent = sendfile(fd, sock, offset, chunk, NULL, NULL, 0)) < 0) {
 #endif
 		log("sendfile: %s", strerror(errno));
 		if (lfd != -1)
@@ -290,6 +291,10 @@ static void handle_get(int sock, char *request, int fd)
 		return;
 	}
 
+	/* The client asked for more data than we have, send padding bytes */
+	while (sent++ < chunk)
+		write(sock, "\0", 1);
+
 	if (lfd != -1)
 		close(lfd);
 }
