@@ -35,7 +35,9 @@
 #include <sys/wait.h>
 #include <sys/select.h>
 #include <sys/types.h>
+#include <fcntl.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "kissd.h"
 
@@ -76,10 +78,8 @@ static int parse_config(void)
 		}
 	}
 	else {
-		if ((home = getenv("HOME")) == NULL) {
-			perror("getenv");
-			return 1;
-		}
+		if ((home = getenv("HOME")) == NULL)
+			home = "/";
 		snprintf(buffer, sizeof(buffer), "%s/.kissd.conf", home);
 		configpath = buffer;
 		if ((f = fopen(configpath, "r")) == NULL) {
@@ -168,12 +168,12 @@ static int parse_config(void)
 	return 0;
 }
 
-static void create_socket(int *sock, int port) {
+static void create_socket(int *sock, int port, int type) {
 	struct sockaddr_in sin;
 	int yes = 1;
 
 	/* get an internet domain socket */
-	if ((*sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	if ((*sock = socket(AF_INET, type, 0)) < 0) {
 		log("socket: %s", strerror(errno));
 		exit(1);
 	}
@@ -196,6 +196,11 @@ static void create_socket(int *sock, int port) {
 		exit(1);
 	}
 
+	/* everything done for a udp socket */
+	if (type == SOCK_DGRAM) {
+		return;
+	}
+
 	/* show that we are willing to listen */
 	if (listen(*sock, 5) < 0) {
 		log("listen: %s", strerror(errno));
@@ -203,10 +208,46 @@ static void create_socket(int *sock, int port) {
 	}
 }
 
+
+static void udp_responder (int sd) {
+	struct sockaddr_in sin;
+	socklen_t sinlen = sizeof(sin);
+	char buf[128];
+
+	/* complete the socket structure */
+	memset(&sin, 0, sizeof(sin));
+	sin.sin_family = AF_INET;
+	sin.sin_addr.s_addr = INADDR_ANY;
+	sin.sin_port = htons(PORT_PCLINK);
+
+	if (recvfrom(sd, buf, sizeof(buf), 0, (struct sockaddr*)&sin, &sinlen) < 0) {
+		log("recvfrom: %s", strerror(errno));
+		return;	// exit gracefully
+	}
+
+	if (strstr(buf, "ARE_YOU_KISS_PCLINK_SERVER?")) {
+		char hostname[255];
+
+		log("Received identity request from %s", inet_ntoa(sin.sin_addr));
+
+		if (gethostname(hostname, sizeof(hostname)) < 0) {
+			log("gethostname: %s", strerror(errno));
+			return; // exit gracefully
+		}
+		strcat(hostname, " running kissd-");
+		strcat(hostname, KISSD_VERSION);
+
+		if (sendto(sd, hostname, strlen(hostname), 0, (struct sockaddr*)&sin, sizeof(sin)) < 0) {
+			log("sendto: %s", strerror(errno));
+		}
+	}
+}
+
 static void do_daemon(void)
 {
 	int sd_server, sd_client;
 	int sd_pclink, sd_kml = -1;
+	int sd_udp_responder;
 	struct sockaddr_in pin;
 	socklen_t addrlen;
 	fd_set set;
@@ -218,26 +259,35 @@ static void do_daemon(void)
 		openlog("kissd", LOG_PID, LOG_DAEMON);
 	log("%s", "starting kissd...");
 
-	create_socket(&sd_pclink, PORT_PCLINK);
+	create_socket(&sd_pclink, 	 PORT_PCLINK, SOCK_STREAM);
+	create_socket(&sd_udp_responder, PORT_PCLINK, SOCK_DGRAM);
+
 	if (k_opt)
-		create_socket(&sd_kml, PORT_KML);
+		create_socket(&sd_kml, PORT_KML, SOCK_STREAM);
 
 again:
 	FD_ZERO(&set);
 	FD_SET(sd_pclink, &set);
+	FD_SET(sd_udp_responder, &set);
+
 	if (k_opt)
 		FD_SET(sd_kml, &set);
 
 	memset(&pin, 0, sizeof(pin));
 	memset(&addrlen, 0, sizeof(addrlen));
 
-	if (select(max(sd_pclink, sd_kml) + 1, &set, NULL, NULL, NULL) < 0) {
+	if (select(max(max(sd_pclink, sd_kml), sd_udp_responder) + 1, &set, NULL, NULL, NULL) < 0) {
 		if (errno == EINTR)
 			goto again;
 		log("select: %s", strerror(errno));
 		exit(1);
 	}
 
+	if (FD_ISSET(sd_udp_responder, &set)) {
+		udp_responder(sd_udp_responder);
+		goto again;
+	}
+
 	if (FD_ISSET(sd_pclink, &set))
 		sd_server = sd_pclink;
 	else
@@ -290,7 +340,7 @@ static void usage(const char *program)
 
 int main(int argc, char *argv[])
 {
-	char c;
+	int c;
 	static struct option long_options[] = {
 		{ "config", 1, NULL, 'c' },
 		{ "daemon", 0, NULL, 'd' },
